import json
import logging
from urllib.parse import urlencode

import requests
from django.conf import settings

from unicef_vision.exceptions import VisionException
from unicef_vision.settings import TIMEOUT
from unicef_vision.utils import base_headers

logger = logging.getLogger(__name__)

INSIGHT_NO_DATA_MESSAGE = 'No Data Available'


class VisionDataLoader:
    """Base class for Data Loading"""

    def __init__(self, endpoint, detail=None, **kwargs):

        self.URL = kwargs.get('url', settings.INSIGHT_URL)
        self.set_headers(kwargs.get('headers', ()))
        querystring = urlencode(kwargs)
        self.set_url(endpoint, detail, querystring)

    def set_url(self, endpoint, detail, querystring):
        separator = '' if self.URL.endswith('/') else '/'

        self.url = '{}{}{}'.format(self.URL, separator, endpoint)
        if detail:
            self.url += '/{}'.format(detail)
        if querystring:
            self.url += '/?{}'.format(querystring)
        logger.info('About to get data from {}'.format(self.url))

    def set_headers(self, headers):
        self.headers = base_headers
        if headers:
            for header_name, header_value in headers:
                self.headers[header_name] = header_value

    def get(self):
        response = requests.get(
            self.url,
            headers=self.headers,
            timeout=TIMEOUT
        )

        if response.status_code != 200:
            raise VisionException('Load data failed! Http code: {}'.format(response.status_code))
        json_response = response.json()
        if json_response == INSIGHT_NO_DATA_MESSAGE:
            return []

        return json_response


class FileDataLoader:
    """Loader to read json file instead of REST API"""

    def __init__(self, filename, detail=None, **kwargs):
        self.filename = filename
        self.detail = detail

    def get(self):
        data = json.load(open(self.filename))
        return data



import requests
from django.apps import apps
from django.conf import settings

from unicef_vision.settings import TIMEOUT

base_headers = {'Content-Type': 'application/json', 'Ocp-Apim-Subscription-Key': settings.INSIGHT_SUB_KEY}


def get_vision_logger_domain_model():
    get_model = apps.get_model
    return get_model(settings.INSIGHT_LOGGER_MODEL)


def get_data_from_insight(endpoint, data=None):
    separator = '' if settings.INSIGHT_URL.endswith('/') else '/'

    if not data:
        data = {}

    url = '{}{}{}'.format(
        settings.INSIGHT_URL,
        separator,
        endpoint
    ).format(**data)

    resp = requests.get(
        url,
        headers=base_headers,
        timeout=TIMEOUT
    )
    if resp.status_code != 200:
        return False, 'Loading data from Vision Failed, status {}'.format(resp.status_code)
    try:

        result = resp.json()
    except ValueError:
        return False, 'Loading data from Vision Failed, no valid response returned for data: {}'.format(data)
    return True, result


def comp_decimals(y, x):
    def isclose(a, b, rel_tol=1e-09, abs_tol=0.0):
        return abs(a - b) <= max(rel_tol * max(abs(a), abs(b)), abs_tol)

    return isclose(float(x), float(y))
